@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2022 Live Networks, Inc.  All rights reserved.
 // A generic media server class, used to implement a RTSP server, and any other server that uses
 //  "ServerMediaSession" objects to describe media to be served.
 // Implementation
@@ -126,8 +126,8 @@ GenericMediaServer
     fServerMediaSessions(HashTable::create(STRING_HASH_KEYS)),
     fClientConnections(HashTable::create(ONE_WORD_HASH_KEYS)),
     fClientSessions(HashTable::create(STRING_HASH_KEYS)),
-    fPreviousClientSessionId(0)
-{
+    fPreviousClientSessionId(0),
+    fTLSCertificateFileName(NULL), fTLSPrivateKeyFileName(NULL) {
   ignoreSigPipeOnSocket(fServerSocketIPv4); // so that clients on the same host that are killed don't also kill us
   ignoreSigPipeOnSocket(fServerSocketIPv6); // ditto
   
@@ -142,6 +142,8 @@ GenericMediaServer::~GenericMediaServer() {
   ::closeSocket(fServerSocketIPv4);
   envir().taskScheduler().turnOffBackgroundReadHandling(fServerSocketIPv6);
   ::closeSocket(fServerSocketIPv6);
+
+  delete[] fTLSCertificateFileName; delete[] fTLSPrivateKeyFileName;
 }
 
 void GenericMediaServer::cleanup() {
@@ -249,15 +251,32 @@ void GenericMediaServer::incomingConnectionHandlerOnSocket(int serverSocket) {
   (void)createNewClientConnection(clientSocket, clientAddr);
 }
 
+void GenericMediaServer
+::setTLSFileNames(char const* certFileName, char const* privKeyFileName) {
+  delete[] fTLSCertificateFileName; fTLSCertificateFileName = strDup(certFileName);
+  delete[] fTLSPrivateKeyFileName; fTLSPrivateKeyFileName = strDup(privKeyFileName);
+}
+
 
 ////////// GenericMediaServer::ClientConnection implementation //////////
 
 GenericMediaServer::ClientConnection
-::ClientConnection(GenericMediaServer& ourServer, int clientSocket, struct sockaddr_storage const& clientAddr)
-  : fOurServer(ourServer), fOurSocket(clientSocket), fClientAddr(clientAddr) {
+::ClientConnection(GenericMediaServer& ourServer,
+		   int clientSocket, struct sockaddr_storage const& clientAddr,
+		   Boolean useTLS)
+  : fOurServer(ourServer), fOurSocket(clientSocket), fClientAddr(clientAddr), fTLS(envir()) {
   // Add ourself to our 'client connections' table:
   fOurServer.fClientConnections->Add((char const*)this, this);
   
+  if (useTLS) {
+    // Perform extra processing to handle a TLS connection:
+    fTLS.setCertificateAndPrivateKeyFileNames(ourServer.fTLSCertificateFileName,
+					      ourServer.fTLSPrivateKeyFileName);
+    fTLS.isNeeded = True;
+
+    fTLS.tlsAcceptIsNeeded = True; // call fTLS.accept() the next time the socket is readable
+  }
+
   // Arrange to handle incoming requests:
   resetRequestBuffer();
   envir().taskScheduler()
@@ -285,9 +304,21 @@ void GenericMediaServer::ClientConnection::incomingRequestHandler(void* instance
 }
 
 void GenericMediaServer::ClientConnection::incomingRequestHandler() {
-  struct sockaddr_storage dummy; // 'from' address, meaningless in this case
+  if (fTLS.tlsAcceptIsNeeded) { // we need to successfully call fTLS.accept() first:
+    if (fTLS.accept(fOurSocket) <= 0) return; // either an error, or we need to try again later
+
+    fTLS.tlsAcceptIsNeeded = False;
+    // We can now read data, as usual:
+  }
+
+  int bytesRead;
+  if (fTLS.isNeeded) {
+    bytesRead = fTLS.read(&fRequestBuffer[fRequestBytesAlreadySeen], fRequestBufferBytesLeft);
+  } else {
+    struct sockaddr_storage dummy; // 'from' address, meaningless in this case
   
-  int bytesRead = readSocket(envir(), fOurSocket, &fRequestBuffer[fRequestBytesAlreadySeen], fRequestBufferBytesLeft, dummy);
+    bytesRead = readSocket(envir(), fOurSocket, &fRequestBuffer[fRequestBytesAlreadySeen], fRequestBufferBytesLeft, dummy);
+  }
   handleRequestBytes(bytesRead);
 }
 
